@@ -29,7 +29,7 @@ if len(R_HOME) == 0:
                                          0, win32con.KEY_QUERY_VALUE )
             R_HOME = win32api.RegQueryValueEx(hkey, "InstallPath")[0]
             win32api.RegCloseKey( hkey )
-        except ImportError, ie:
+        except ImportError(ie):
             raise RuntimeError(
                 "No environment variable R_HOME could be found, "
                 "calling the command 'R RHOME' does not return anything, " +\
