@@ -54,7 +54,7 @@ NULL = robjects.NULL
 
 ggplot2 = importr('ggplot2')
 
-TARGET_VERSION = '0.9.3.1'
+TARGET_VERSION = '1.0.0'
 if ggplot2.__version__ != TARGET_VERSION:
    warnings.warn('This was designed againt ggplot2 version %s but you have %s' % (TARGET_VERSION, ggplot2.__version__))
 ggplot2_env = robjects.baseenv['as.environment']('package:ggplot2')
@@ -107,7 +107,7 @@ class Aes(robjects.Vector):
     def new(cls, **kwargs):
         """Constructor for the class Aes."""
         new_kwargs = copy.copy(kwargs)
-        for k,v in kwargs.iteritems():
+        for k,v in kwargs.items():
             new_kwargs[k] = as_symbol(v)
         res = cls(cls._constructor(**new_kwargs))
         return res
@@ -133,7 +133,7 @@ class AesString(robjects.Vector):
     def new(cls, **kwargs):
        """Constructor for the class AesString."""
        new_kwargs = copy.copy(kwargs)
-       for k,v in kwargs.iteritems():
+       for k,v in kwargs.items():
           new_kwargs[k] = as_symbol(v)
        res = cls(cls._constructor(**new_kwargs))
        return res
@@ -680,6 +680,15 @@ class ElementRect(Element):
        return res
 element_rect = ElementRect.new
 
+class Labs(Options):
+   _constructor = ggplot2.labs
+   @classmethod
+   def new(cls, **kwargs):
+      res = cls(cls._constructor(**kwargs))
+      return res
+
+labs = Labs.new
+
 class Theme(Options):
    pass
 
@@ -783,19 +792,22 @@ theme = ggplot2_env['theme']
 
 ggtitle = ggplot2.ggtitle
 
-original_conversion = conversion.ri2py
+original_ri2ro = conversion.ri2ro
 def ggplot2_conversion(robj):
 
-    pyobj = original_conversion(robj)
+    pyobj = original_ri2ro(robj)
 
-    rcls = pyobj.rclass
-    if rcls is NULL:
-       rcls = (None, )
+    try:
+       rcls = pyobj.rclass
+    except AttributeError:
+       # conversion lead to something that is no
+       # longer an R object
+       return pyobj
 
-    if 'gg' in rcls:
+    if (rcls is not None) and ('gg' in rcls):
        pyobj = GGPlot(pyobj)
 
     return pyobj
 
-conversion.ri2py = ggplot2_conversion
+conversion.ri2ro = ggplot2_conversion
 
